// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

const GET_ACCOUNT_URL = "http://flock.com/favorites-services";

Components.utils.import("resource:///modules/FlockOnlineFavsUtils.jsm");

var faves_coop = Components.classes['@flock.com/singleton;1']
                           .getService(Components.interfaces.flockISingleton)
                           .getSingleton("chrome://flock/content/common/load-faves-coop.js")
                           .wrappedJSObject;

var doEdited = false;
var onlineAccounts = null;

var Logger = Components.Constructor("@flock.com/logger;1", Components.interfaces.flockILogger, "init");

var gMetrics = Components.classes["@flock.com/metrics-service;1"]
               .getService(Components.interfaces.flockIMetricsService);

var progressListener = {
  _logger: new Logger("webprogress"),
  lastUrl: null,
  STATE_STOP: Components.interfaces.nsIWebProgressListener.STATE_STOP,
  STATE_IS_NETWORK: Components.interfaces.nsIWebProgressListener.STATE_IS_NETWORK,
  STATE_START: Components.interfaces.nsIWebProgressListener.STATE_START,
  mButton: null,
  currentUrl: null,
  
  onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) { },
  
  onProgressChange: function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) { },

  onLocationChange: function(aWebProgress, aRequest, aLocation) {
    var url = aLocation.spec;
    // strip anchors
    url = url.replace(/#.*/, '');
    if (this.lastUrl != url) {
      this._newLocation(url);
      this.lastUrl = url;
    }
  },
  
  onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) { },

  onSecurityChange: function(aWebProgress, aRequest, aStatus) { },

  _updateButton: function() {
      updateState();
  },

  _newLocation: function(aUrl) {
    var nBox = gBrowser.getNotificationBox();
    nBox.clearDiscoveryMessages();
    this.currentUrl = aUrl;
    updateLastVisitedDate(aUrl); 
    this._updateButton();
  }
};


/*
 * Start update Favorite and Explorer button
 *
 */

var publishListener = {
  onError: function() {
    var message = flockGetString("favorites/favorites",
                                 "flock.favs.overlay.favorite-publication-error",
                                 [bookmark.URL, service[0]]);
    var nBox = gBrowser.getNotificationBox();
    nBox.appendUniqueNotification(message,
                                  "favorite-publication-error",
                                  "chrome://flock/skin/common/Info.png",
                                  nBox.FLOCK_PRIORITY_HIGH,
                                  null);
  },
  onSuccess: function() {
    updateState();
  }
}

function updateState() {
  var docshell= top.document.getElementById("content").webNavigation;
  updateStarButton(docshell);
}

function updateStarStateForBookmark() {
}

function updateStarButton(docshell)
{
  updateStarStateForBookmark(docshell);
}


function editCurrentBookmark(URL) {
  // if star is waiting we do not want to open dialog
  if (starButton.isStarWaiting()) {
    return;
  }

  if (!URL || typeof(URL) != "string") {
    URL = gBrowser.currentURI.spec;
  }

  var coopObj = faves_coop.Bookmark.get_from_url(URL);

  if (doEdited) {
    starButton.waitCursor('on');
    setTimeout(
      function() {
        openFavoritesEditDialog(coopObj);
        starButton.waitCursor('off');
      },
      3000);
  } else {
    openFavoritesEditDialog(coopObj);
  }
}

function openFavoritesEditDialog(aCoopObj) {
  var selection = { item: [ { Value: aCoopObj.id() } ] };
  var retval = {};

  var windowMediator = Components
                       .classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);

  // See if the dialog is not already open
  var oldWin = null;
  var allWindows = windowMediator.getEnumerator(null);
  while (allWindows.hasMoreElements()) {
    var win = allWindows.getNext();
    if (win.arguments && win.arguments.length > 0 &&
        win.arguments[0] &&
        win.arguments[0].item == selection.item)
    {
      oldWin = win;
      break;
    }
  }

  if (oldWin) {
    oldWin.focus();
  } else {
    // bookmarksProperties.xul takes a menuitem/treeitem instead of a URL
    // (yes, that sucks)

    openDialog("chrome://browser/content/bookmarks/bookmarksProperties.xul",
               "", "centerscreen,modal,resizable=no",
               selection, retval);
  }

  doEdited = retval.publishedOnline;
}

function starClicked(aTriggeringEvent)
{

//@line 178 "/home/build/tinderbuild_red/src/flock/mozilla/flock/base/favorites/content/favoritesOverlay.js"
 var accel = aTriggeringEvent.ctrlKey;
//@line 180 "/home/build/tinderbuild_red/src/flock/mozilla/flock/base/favorites/content/favoritesOverlay.js"
  if(accel)
  {
    var URL = gBrowser.currentURI.spec;
    if (favoritesService.isBookmarked (URL)) {
      editCurrentBookmark();
    }else
    {
      var name = gBrowser.contentTitle;
      var description = BookmarksUtils.getDescriptionFromDocument(gBrowser.contentDocument);
      var charset = gBrowser.contentDocument.characterSet;
      showAddBookmarksDialog(name, URL, description, null, charset);
    }
  }else
  {
    starCurrentPage();
  }
}

function starCurrentPage(dArgs) {
  var starStared = false;
  if (starButton.getStarState() == "off") {
    starStared = true;
  }
  gMetrics.report("Navigation-StarClick", starStared);

  var URL = null;
  if (dArgs && dArgs.url) {
    URL = dArgs.url;
  }else
  {
    URL = gBrowser.currentURI.spec;
  }
  if (gBrowser.currentURI.spec == URL &&
      starButton.getStarState() == "on2")
  {
    showOnlineBookmarkProperties(URL);
  }
  else if (favoritesService.isBookmarked (URL)) {
    if (dArgs) {
      editCurrentBookmark(URL);
    } else {
      editCurrentBookmark();
    }
  }
  else {
    var name = gBrowser.contentTitle;
    var description = BookmarksUtils
                      .getDescriptionFromDocument(gBrowser.contentDocument);
    var charset = gBrowser.contentDocument.characterSet;
    if (dArgs) {
      name = dArgs.name;
      description = dArgs.description;
    }
    // Does the user want to get the dialog?
    var prefsService = Components.classes["@mozilla.org/preferences-service;1"]
                                 .getService(Components.interfaces.nsIPrefBranch);
    var starShowProperties = prefsService
                             .getBoolPref("flock.favorites.starShowProperties");
    if (starShowProperties) {
      // Popup the new BM dialog
      showAddBookmarksDialog(name, URL, description, null, charset);
    }
    else {
      // Create the bookmark and inform the user
      if (prefsService.getBoolPref("flock.favorites.do.local"))
      {
        var folder = prefsService.getCharPref("flock.favorites.do.folder");
        if(folder != '')
        {
          BMDS.addBookmarkToFolder(folder, name, URL, "", description, charset, "", "");
        }else
        {
          BMDS.createBookmarkWithTags (name, URL, "", description, charset, "", "");
        }
      }
      var prefBranch = prefsService.getBranch("flock.favorites.do.");
      var list = prefBranch.getChildList("", {});
      var bookmark = new bookmarksHelper.Bookmark;
      bookmark.URL = URL;
      bookmark.name = name;
      bookmark.tags = "";
      bookmark.description = "";
      
      var publishList = [];
      for (var i =0; i<list.length; i++) {
        try {
          if (prefsService.getBoolPref("flock.favorites.do."+list[i])) {
            var isPrivate = false;
            try {
                isPrivate = prefsService.getBoolPref("flock.favorites.private." + list[i]);
            } catch (ex) {
            }
            var service = list[i].split("--");
            if(service.length == 2) {
              var svc = Cc[service[0]].getService(Ci.flockIBookmarkWebService);
              svc.publish(publishListener, service[1], bookmark, isPrivate);
              var found = false;
              for (var k = 0; k < publishList.length; k++) {
                if (publishList[k] == list[i]) {
                  found = true;
                  break;
                }
              }
              if (!found) {
                publishList.push(list[i]);
              }
              doEdited = true;
            }
          }
        }catch(e) {}
      }
      
      showStarMessage(prefsService.getBoolPref("flock.favorites.do.local"), publishList);
    }
  }
  updateStarStateForBookmark();
}

function showOnlineBookmarkProperties(url)
{
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                          .getService(Ci.flockIAccountUtils);
  var accounts = acUtils.getAccountsByInterface("flockIBookmarkWebService");
  var bookmark = null;
  while (accounts.hasMoreElements()) {
    var next_acc = accounts.getNext();
    var account = next_acc.QueryInterface(Ci.flockIWebServiceAccount);
    var c_acct = faves_coop.get(account.urn);
    var svc = Cc[c_acct.serviceId].getService(Ci.flockIWebService);
    try {
      svc.QueryInterface(Ci.flockIBookmarkWebService);
      bookmark = faves_coop.get(svc.urn+":"+c_acct.accountId+':'+url);
      if (bookmark) {
        // found a online bookmark for this URL
        break;
      }
    } catch(ex) {
    }
  }
  if (bookmark) {
    showAddBookmarksDialog(bookmark.name, url, bookmark.description,
                           bookmark.tags);
  }
}

function showAddBookmarksDialog(name, url, description, tags, charset, dArgs) {
  dump("****** showAddBookmarksDialog: " + url + "\n");

    var prefsSvc = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
  var defaultFolder = prefsSvc.getCharPref('flock.favorites.do.folder');
  dArgs = {
    name: name,
    url: url,
    bWebPanel: false,
    folderURI: defaultFolder,
    description: description,
    tags: tags,
    charset: charset,
    listener: publishListener
  };
    
  openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable=no,modal", dArgs)
  if ((favoritesService.isBookmarked (url)) || (dArgs.publishList.length > 0)) {
    showStarMessage(favoritesService.isBookmarked (url), dArgs.publishList);
  }
}

function turnOffDiscoveryMessage(aValue) {
  var nBox = gBrowser.getNotificationBox();
  nBox.dontShowMessage(aValue);
}


/*
 * End update Favorite and Explorer button
 *
 */

var favoritesObserverService = Components.classes["@mozilla.org/observer-service;1"]
                                         .getService( Components.interfaces.nsIObserverService );

function updateLastVisitedDate(aUrl)
{
}

var mediaObserver;
var flockDocumentReadyObserver;
var favoritesObserver;
var starButton;
var gTimer = null;

function showMessages(targetDoc)
{
   var browser = gBrowser.getBrowserForDocument(targetDoc);
   var nBox = gBrowser.getNotificationBox(browser);
   nBox.showDiscoveryMessages();
} 

function onLoad_favoritesOverlay() {
  var onlineFolderId = onlineFavoritesBackend.getOnlineRootId();
  var onlinePlace = PlacesUtils.getQueryStringForFolder(onlineFolderId);
  document.getElementById("flockOnlineFolderPopup").place = onlinePlace;

  starButton = document.getElementById("favoritesButton");
    
  mediaObserver = { observe: function (aSubject, aTopic, aData) { updateState(); } };
  favoritesObserverService.addObserver(mediaObserver, 'media', false);

  flockDocumentReadyObserver = { observe: function (aSubject, aTopic, aData) { updateState(); setTimeout(showMessages, 500, aSubject)} };
  favoritesObserverService.addObserver(flockDocumentReadyObserver, 'FlockDocumentReady', false);

  // our general purpose web progress listener, apparently
  progressListener.mButton = starButton;

  window.getBrowser().addProgressListener(progressListener,
                       Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
}

function onUnload_favoritesOverlay() {
  if (favoritesObserver) favoritesService.RemoveObserver(favoritesObserver);
  if (mediaObserver) favoritesObserverService.removeObserver( mediaObserver, 'media' );
  if (flockDocumentReadyObserver) favoritesObserverService.removeObserver( flockDocumentReadyObserver, 'FlockDocumentReady');
  window.getBrowser().removeProgressListener(progressListener);
}

window.addEventListener("load", onLoad_favoritesOverlay, false);
window.addEventListener("unload", onUnload_favoritesOverlay, false);

function populatesFeedFolder (aMenuPopup, aFeedFolder) {
  var feedManager = Components.classes["@flock.com/feed-manager;1"].getService(Components.interfaces.flockIFeedManager);

  // Clean the list
  while(aMenuPopup.childNodes.length > 0)
    aMenuPopup.removeChild(aMenuPopup.lastChild);

  var children = aFeedFolder.getChildren();
  while (children.hasMoreElements()) {
    var folderItem = children.getNext();
    if (folderItem instanceof Components.interfaces.flockIFeedFolder) {
      folderItem.QueryInterface(Components.interfaces.flockIFeedFolder);
      var menu = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menu");
      menu.setAttribute("label", folderItem.getTitle());
      var menupopup = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menupopup");
      menupopup.feedFolder = folderItem;
      menupopup.setAttribute("onpopupshowing", "populatesFeedFolder(this, this.feedFolder);event.stopPropagation()");
      menu.appendChild(menupopup);
      aMenuPopup.appendChild(menu);     
    }
    else if (folderItem instanceof Components.interfaces.flockIFeed) {
      folderItem.QueryInterface(Components.interfaces.flockIFeed);
      var menuitem = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
      menuitem.setAttribute("label", folderItem.getTitle());
      if (folderItem.getFavicon())
        menuitem.setAttribute("image", folderItem.getFavicon().spec);
      menuitem.setAttribute("url", folderItem.getURL().spec);
      menuitem.setAttribute("oncommand", "FlockFeeds.openSidebar('FeedsMenuItemClick'); "
                            + "openUILink('flock://favorites/urn:flock:feed:' +"
                            + " this.getAttribute('url')); "
                            + "event.stopPropagation();");
      aMenuPopup.appendChild(menuitem);     
    }
  }
  
  BookmarksMenu.showEmptyItem(aMenuPopup);
}

function populatesFeedContext (aMenuPopup) {
  var feedManager = Components.classes["@flock.com/feed-manager;1"].getService(Components.interfaces.flockIFeedManager);

  var news = feedManager.getFeedContext("news");
  populatesFeedFolder(aMenuPopup, news.getRoot());
}

function publishCurrent(accountId, serviceId) {
  var svc = Cc[serviceId].getService(Ci.flockIBookmarkWebService);

  var bookmark = new Bookmark;
  bookmark.URL = gBrowser.currentURI.spec;
  bookmark.name = gBrowser.contentTitle;

  var listener = {
    onSuccess: function (aSubject, aTopic) {
      updateState();
    },
    onError: function (aFlockError, aTopic) {
      var msg = flockGetString("favorites/favorites", "flock.favs.overlay.publish.error");
      alert(msg);
    }
  }

  svc.publish(listener, accountId, bookmark, false);
}

function favoritesOverlay_showEmptyItem(aEvent) {
  var popup = aEvent.target;
  var emptyCount = 0;
  if (popup.id == "mediabar-favorites-popup") {
    // The template node is present, so an empty popup has one child
    emptyCount = 1;
  }

  if (popup.childNodes.length <= emptyCount) {
    var empty = document.createElement("menuitem");
    var label = PlacesUIUtils.getString("bookmarksMenuEmptyFolder");
    empty.setAttribute("label", label);
    empty.setAttribute("disabled", true);
    popup.appendChild(empty);
  } else {
    var children = popup.childNodes;
    for (var i = 0; i < children.length; i++) {
      child = children[i];
      if (child.getAttribute("disabled")) {
        popup.removeChild(child);
      }
    }
  }
}

function debug(aStr) {
  dump('favoritesOverlay.js ' + aStr + '\n');
}
