// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

Components.utils.import("resource:///modules/FlockStringBundleHelpers.jsm");

var mediabar = {
  _favsCoop : Components.classes['@flock.com/singleton;1']
                           .getService(Components.interfaces.flockISingleton)
                           .getSingleton("chrome://flock/content/common/load-faves-coop.js")
                           .wrappedJSObject,
  initialMedia: null,
  load: function(aAccount) {
    var acctCoop = mediabar._favsCoop.get(aAccount.urn);
    if (!acctCoop) {
      // Invalid account
      return;
    }
    var svc = aAccount.getService();
    if (!(svc instanceof Components.interfaces.flockIMediaWebService)) {
      // Invalid service
      return;
    }

    if (FlockTopbar.JSCtx().gPhotoDrawer) {
      // The mediabar is already open
      FlockTopbar.JSCtx().gPhotoDrawer.loadPhotoPerson(aAccount.urn, svc.shortName);
    }
    else {
      // The mediabar is closed; need to open it
      this.initialMedia = {
        person: {
          urn: aAccount.urn,
          svc: svc.shortName
        }
      }
      mediabar.ensureOpen();
    }
  },
  loadQuery: function(aService, aQuery, aName) {
    if (FlockTopbar.JSCtx().gPhotoDrawer) {
      FlockTopbar.JSCtx().gPhotoDrawer.mFilter = '';
      FlockTopbar.JSCtx().gPhotoDrawer.loadQuery( aService, aQuery, aName );
      FlockTopbar.JSCtx().gPhotoDrawer.emptySearchbox();
    }
    else {
      this.initialMedia = {
        query: {
          service: aService,
          query: aQuery,
          name: aName
        }
      }
      mediabar.ensureOpen();
    }
  },
  loadFromId: function(aId) {
    if (FlockTopbar.JSCtx().gPhotoDrawer) {
      FlockTopbar.JSCtx().gPhotoDrawer.loadFromId( aId );
    }
    else {
      var item = mediabar._favsCoop.get(aId);
      this.initialMedia = {
        query: {
          service: item.service,
          query: item.query,
          name: item.name
        }
      }
      mediabar.ensureOpen();
    }
  },
  showActionMessage: function(aMessage) {
    mediabar.ensureOpen();
    if (FlockTopbar.JSCtx().gPhotoDrawer) {
      FlockTopbar.JSCtx().gPhotoDrawer.showActionMessage(aMessage);
    } else {
      // The mediabar is not initialized yet:
      // set a variable that will be picked up at initialization
      FlockTopbar.JSCtx().gInitialActionMessage = aMessage;
    }
  },
  hideActionMessage: function() {
    if (FlockTopbar.JSCtx().gPhotoDrawer) {
      FlockTopbar.JSCtx().gPhotoDrawer.hideActionMessage();
    }
  },
  lock: function()
  {
    mediabar.ensureOpen();
    var retval = true;
    if (FlockTopbar.JSCtx().gPhotoDrawer) {
      retval = FlockTopbar.JSCtx().gPhotoDrawer.lock();
    }
    return retval;
  },
  unlock: function()
  {
    if (FlockTopbar.JSCtx().gPhotoDrawer)
    {
      FlockTopbar.JSCtx().gPhotoDrawer.unlock();
    }
  },
  ensureOpen: function() {
    FlockTopbar.selectById('flock_MediabarTopbarBroadcaster', true)
  },
  needDiscovery: function(aBrowser, aService, aQuery, aUrl)
  {
    var showDiscovery = true;
   // if the topbar is open with mediabar
    if (document.getElementById("topbarBrowser").getAttribute("src") &&
        FlockTopbar.JSCtx().gPhotoDrawer)
    {
      if ((FlockTopbar.JSCtx().gPhotoDrawer.mApi == aService) &&
          (FlockTopbar.JSCtx().gPhotoDrawer.mQuery.stringVal == aQuery))
      {
        showDiscovery = false;
        var nBox = aBrowser.getNotificationBox();
        nBox.saveToHashtable(aUrl, nBox.FLOCK_PRIORITY_LOW);
      }
    }
    return showDiscovery;
  },
  openViaDiscovery: function mediabar_openViaDiscovery(aMenuitem) {
    var service = aMenuitem.getAttribute("service");
    var query = aMenuitem.getAttribute("query");
    var label = aMenuitem.getAttribute("label");

    aMenuitem.parentNode.hidePopup();
    mediabar._openViaItems(service, query, label);
  },
  _openViaItems: function mediabar__openViaItems(aService, aQuery, aLabel) {
    if (mediabar.lock()) {
      gMetrics.report("MediaBar-Open", "Discovery");
      mediabar.loadQuery(aService, aQuery, aLabel);
      mediabar.unlock();

      // Pop message bar for media detection discovery
      var favItem = faves_coop.MediaQuery.find({ query: aQuery,
                                               isPollable: true });
      if (favItem.length == 0) {
        var msg = flockGetString("favorites/favorites",
                                "flock.favs.media.discovery");
        var nBox = notificationOverlay.getNotificationBox();
        var mediaIcon = "chrome://flock/skin/favorites/media-icon.png";
        nBox.appendUniqueNotification(msg,
                                      "media-discovery-info",
                                      mediaIcon,
                                      nBox.FLOCK_PRIORITY_LOW,
                                      null,
                                      null);
      }
    }
  }
};

var MediaNotifications = {
};

MediaNotifications.init =
function MN_init() {
  gBrowser.addEventListener("TabSelect",
                            MediaNotifications.updateState,
                            false);
};

MediaNotifications.uninit =
function MN_uninit() {
  gBrowser.removeEventListener("TabSelect",
                               MediaNotifications.updateState,
                               false);
};

MediaNotifications.updateState =
function MN_updateState() {
  // Check for discovered Media
  var mediaBtn = document.getElementById("favoritesPhotoButton");
  var docshell = top.document.getElementById("content").webNavigation;
  var decorations = docshell.document._flock_decorations;
  if (decorations && decorations.mediaArr && decorations.mediaArr.length) {
    // Media was discovered
    mediaBtn.removeAttribute("disabled");
    mediaBtn.setAttribute("tooltiptext",
                          flockGetString("favorites/favorites",
                                         "flock.favs.overlay.mediaBtn.new"));
    MediaNotifications._showMessage(decorations);
  } else {
    // No media was discovered
    mediaBtn.setAttribute("disabled", "true");
    mediaBtn.setAttribute("tooltiptext",
                          flockGetString("favorites/favorites",
                                         "flock.favs.overlay.mediaBtn.desc"));
  }
};

MediaNotifications._showMessage =
function MN__showMessage(aDecorations) {
  var nBox = gBrowser.getNotificationBox();
  if (aDecorations && aDecorations.mediaArr) {
    for (var i = 0; i < aDecorations.mediaArr.length; i++) {
      var mStream = aDecorations.mediaArr[i];
      var url = mStream.service + "+" + mStream.query;
      if (mStream &&
          mediabar.needDiscovery(gBrowser, mStream.service,
                                 mStream.query, url))
      {
        // Show message bar
        var message = flockGetString("favorites/favorites",
                                     "flock.favs.overlay.discovery-media");
        var links = [{
          label: flockGetString("favorites/favorites",
                                "flock.favs.overlay.show-me-how"),
          callback: function MN_showMessage_callback() {
            var method = "notificationOverlay.flash(1, "
                       + "'favoritesPhotoButton', "
                       + "document.getElementById('favoritesPhotoButton'));";
            setTimeout(method, 500);
            document.getElementById("favoritesPhotoButton").click();
          }
        }];
        nBox.appendUniqueNotification(message,
                                      "discovery-media",
                                      null,
                                      nBox.FLOCK_PRIORITY_LOW,
                                      null,
                                      links,
                                      url);
        break;
      }
    }
  }
};

MediaNotifications.updatePopup =
function MN_updatePopup(aDocshell) {
  var mediaBtn = document.getElementById("favoritesPhotoButton");
  var mediaPopup = mediaBtn.firstChild;
  var rtval_firstquery = null;
  // Clear out the popup menu
  while (mediaPopup.firstChild) {
    mediaPopup.removeChild(mediaPopup.firstChild);
  }
  // Repopulate the menu based on document decorations
  var docshell = (aDocshell) ? aDocshell : top.document
                                              .getElementById("content")
                                              .webNavigation;
  var decorations = docshell.document._flock_decorations;
  if (decorations && decorations.mediaArr) {
    // Only keep media RSS if there is no other media
    var keepMediaRSS = true;
    for (var i = 0; i < decorations.mediaArr.length && keepMediaRSS; i++) {
      if (decorations.mediaArr[i].service != "mediarss") {
        keepMediaRSS = false;
      }
    }
    for (var i = 0; i < decorations.mediaArr.length; i++) {
      var mStream = decorations.mediaArr[i];
      if (!keepMediaRSS && mStream.service == "mediarss") {
        continue;
      }
      if (mStream) {
        var menuitem = document.createElement("menuitem");
        menuitem.setAttribute("service", mStream.service);
        menuitem.setAttribute("discovered", "false");
        menuitem.setAttribute("query", mStream.query);
        menuitem.setAttribute("label", mStream.label);
        menuitem.setAttribute("oncommand", "mediabar.openViaDiscovery(this);");
        var urn = "urn:media:favorites:"+mStream.service+":"+mStream.query;
        if (faves_coop.get(urn)) {
          menuitem.setAttribute("class", "menuitem-iconic");
          menuitem.setAttribute("image",
                                "chrome://flock/skin/common/star16.png");
        }
        mediaPopup.appendChild(menuitem);
        if (!rtval_firstquery) {
          rtval_firstquery = mStream.query;
        }
      }
    }
  }
  // Return value will be NULL if no media was found, else it will be the
  // query string for the first discovered media stream
  return rtval_firstquery;
};

window.addEventListener("load", MediaNotifications.init, false);
window.addEventListener("unload", MediaNotifications.uninit, false);
