// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005-2008
// http://flock.com
//
// This file may be used under the terms of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL

/**************************************************************************
 * Imports
 **************************************************************************/
Components.utils.import("resource://gre/modules/JSON.jsm");

const PREF_DIGG_ENABLE_FLYOUT = "flock.service.digg.enable_flyout";
const PREF_DIGG_HIGHLIGHT_THRESHOLD = "flock.service.digg.highlightThreshold";
const MIN_THRESHOLD = 1000;

var diggPopup = {
  mDiggPopup: null,
  mLogger: null,
  mDiggService: null, 
  mSb: null,
  mMetrics: null,

  POPULAR_THRESHOLD: 5000,

  _init: function diggPopup__init() {
    if (!document.getElementById("digg-button")) {
      return;
    }
    getBrowser().addProgressListener(this, CI.nsIWebProgress.NOTIFY_LOCATION);
    this.mLogger = CC["@flock.com/logger;1"].createInstance(CI.flockILogger);
    this.mLogger.init("diggPopup");
    this.mMetrics = CC["@flock.com/metrics-service;1"]
                    .getService(CI.flockIMetricsService);
    this.mLogger.debug("._init()");
    this.mDiggService = CC["@flock.com/webservice/digg;1"]
                        .getService(CI.flockIWebService);
    this.mPrefBranch = CC["@mozilla.org/preferences-service;1"]
                       .getService(CI.nsIPrefBranch2);
    this.mPrefBranch.addObserver(PREF_DIGG_ENABLE_FLYOUT, this, false);
    this.mPrefBranch.addObserver(PREF_DIGG_HIGHLIGHT_THRESHOLD, this, false);
    this.mSb = CC["@mozilla.org/intl/stringbundle;1"]
               .getService(CI.nsIStringBundleService)
               .createBundle("chrome://flock/locale/people/people.properties");
    this.mIOService = CC["@mozilla.org/network/io-service;1"]
                      .getService(CI.nsIIOService);
    this.mGlobalHistory = CC["@mozilla.org/browser/global-history;2"]
                          .getService(CI.nsIGlobalHistory2);
    var inst = this;
    this.mDiggPopup = document.getElementById("diggPopup");
  },

  _uninit: function diggPopup__uninit() {
    this.mLogger.debug("._uninit()");
    this.mPrefBranch.removeObserver(PREF_DIGG_ENABLE_FLYOUT, this, false);
    this.mPrefBranch.removeObserver(PREF_DIGG_HIGHLIGHT_THRESHOLD, this, false);
  },

  getDiggButton: function diggPopup_getDiggButton() {
    return document.getElementById("digg-button");
  },

  getHighlightThreshold: function diggPopup_getHighlightThreshold() {
    var threshold = this.mPrefBranch
                        .getIntPref(PREF_DIGG_HIGHLIGHT_THRESHOLD);
    if (threshold < MIN_THRESHOLD) {
      return MIN_THRESHOLD;
    } else {
      return threshold;
    }
  },

  observe: function diggPopup_observe(aSubject, aTopic, aData) {
    if (aTopic == "nsPref:changed" &&
         (aData == PREF_DIGG_ENABLE_FLYOUT ||
          aData == PREF_DIGG_HIGHLIGHT_THRESHOLD))
    {
      var enableFlyout = this.mPrefBranch
                             .getBoolPref(PREF_DIGG_ENABLE_FLYOUT);
      if (enableFlyout) {
        this.getDiggButton().removeAttribute("hidden");

        var wm = CC["@mozilla.org/appshell/window-mediator;1"]
                 .getService(CI.nsIWindowMediator);
        var win = wm.getMostRecentWindow("navigator:browser");
        var url = win.document.getElementById("content").currentURI.spec;
        var inst = this;
        var locationChangeListener = {
          QueryInterface: function loc_queryInterface(aIID) {
            if (aIID.equals(CI.nsISupports) ||
                aIID.equals(CI.flockIListener))
            {
              return this;
            }
            throw Components.results.NS_ERROR_NO_INTERFACE;
          },
          onSuccess: function loc_success(aResult) {
            inst.enableDiggMan();
            inst.updateDiggMan(url, aResult);
          },
          onError: function loc_error(aError) {
            inst.disableDiggMan();
          }
        };

        // Bug 13212: load only top stories
        setTimeout(function delayed_getTopStories() {
                     inst.mDiggService.getTopStories(locationChangeListener);
                   },
                   100);
      } else {
        this.getDiggButton().setAttribute("hidden", "true");
      }
    }
  },

  QueryInterface: function diggPopup_QueryInterface(aIID) {
    if (aIID.equals(CC.nsIWebProgressListener) ||
        aIID.equals(CI.nsISupports))
    {
      return this;
    }
    throw Components.results.NS_NOINTERFACE;
  },

  /**
   * void onLocationChange(in nsIWebProgress aProgress,
   *                       in nsIRequest aRequest,
   *                       in nsIURI aURI);
   * @see nsIWebProgressListener#onLocationChange
   */
  onLocationChange: function diggPopup_locationChange(aProgress,
                                                      aRequest,
                                                      aURI)
  {
    var url = aURI.spec.toLowerCase();
    var inst = this;
    var locationChangeListener = {
      QueryInterface: function loc_queryInterface(aIID) {
        if (aIID.equals(CI.nsISupports) ||
            aIID.equals(CI.flockIListener))
        {
          return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
      },
      onSuccess: function loc_success(aResult) {
        inst.enableDiggMan();
        inst.updateDiggMan(url, aResult);
      },
      onError: function loc_error(aError) {
        inst.disableDiggMan();
      }
    };

    if (!this.getDiggButton().hasAttribute("hidden")) {
      // Bug 13212: load only top stories
      // this.mDiggService.loadDiggInfo(url, locationChangeListener, true);
      this.mDiggService.getTopStories(locationChangeListener);
    }
  },

  /** @see nsIWebProgressListener#onProgressChange */
  onProgressChange: function diggPopup_progressChange(aProgress,
                                                      aRequest,
                                                      aCurSelfProgress,
                                                      aMaxSelfProgress,
                                                      aCurTotalProgress,
                                                      aMaxTotalProgress)
  {
  },

  /** @see nsIWebProgressListener#onSecurityChange */
  onSecurityChange: function diggPopup_securityChange(aProgress,
                                                      aRequest,
                                                      aState)
  {
  },

  /** @see nsIWebProgressListener#onStateChange */
  onStateChange: function diggPopup_stateChange(aProgress,
                                                aRequest,
                                                aFlag,
                                                aStatus)
  {
  },

  /** @see nsIWebProgressListener#onStatusChange */
  onStatusChange: function diggPopup_statusChange(aProgress,
                                                  aRequest,
                                                  aStatus,
                                                  aMessage)
  {
  },

  show: function diggPopup_show() {
    if (this.mIsOpen) {
      return;
    }
    if (!document.getElementById("digg-button") ||
        document.getElementById("digg-button").hasAttribute("disabled"))
    {
      return;
    }
    this.mDiggPopup.showPopup(this.getDiggButton(),
                              -1,
                              -1,
                              "popup",
                              "bottomleft",
                              "topleft");
    this.mMetrics.report("DiggManFlyout-Click", null);
  },

  hide: function diggPopup_hide() {
    this.mDiggPopup.hidePopup();
  },

  updateDiggMan: function diggPopup_updateDiggMan(aUrl, aResult) {
    if (!document.getElementById("digg-button")) {
      return;
    }
    aResult.QueryInterface(CI.nsIPropertyBag);
    var shortName = this.mDiggService.shortName;

    // Bug 13212: this function should refresh both top stories and # of
    // diggs for the current page, but now it's only top stories. If you need
    // to reactivate page info, look in SVN for the code that used to be
    // here (around revision 18556)
    document.getElementById("digg-button").removeAttribute("on");

    // Disable the Digg This Story button for non-http pages and pages in
    // digg.com domain.
    if (!(aUrl.match(/^https?:\/\//)) || aUrl.match(/^https?:\/\/digg\.com/)) {
      document.getElementById("diggIt").setAttribute("disabled", "true");
    } else {
      document.getElementById("diggIt").removeAttribute("disabled");
    }

    var topStories = aResult.getProperty("topStories");
    if (topStories) {
      this.loadTopStories(JSON.fromString(topStories));
    }
  },

  loadTopStories: function diggPopup_loadTopStories(aStories) {
    if (!document.getElementById("digg-button")) {
      return;
    }
    this.removeTopStories();
    var topStoriesContainer = document.getElementById("topStoryContent");
    var popular = "false";
    for (var k = 0; k < aStories.length; k++) {
      var story = aStories[k];
      var label = document.createElement("label");
      var id = "topStory_" + story.id;
      label.setAttribute("value", story.title + " (" + story.diggs + " diggs)");
      label.setAttribute("id", id);
      label.setAttribute("src", story.link);
      label.setAttribute("onclick", "diggPopup.performAction('" + id + "');");     
      label.setAttribute("class", "storyLink");

      // Digg url with Flock tracking code
      var linkURI = this.mIOService.newURI(story.link, null, null);
      // Digg url without Flock tracking code
      var origLinkURI = this.mIOService.newURI(story.origLink, null, null);
      // Actual story url
      var realURI = this.mIOService.newURI(story.url, null, null);
      // Only highlight the story as popular if both:
      //   (1) story digg count >= threshold pref value
      //   (2) story has *not* been visited before
      // A Digg story is considered "visited" if any of the 3 above URLs
      // have been visited.
      if (story.diggs >= this.getHighlightThreshold() &&
          !this.mGlobalHistory.isVisited(linkURI) &&
          !this.mGlobalHistory.isVisited(origLinkURI) &&
          !this.mGlobalHistory.isVisited(realURI))
      {
        label.setAttribute("class", "storyLink popularStory");
        popular = "true";
      }
      topStoriesContainer.appendChild(label);
    }
    document.getElementById("digg-button").setAttribute("on", popular);
  },

  removeTopStories: function diggPopup_removeTopStories() {
   var topStoriesContainer = document.getElementById("topStoryContent");
   while (topStoriesContainer.childNodes.length > 1) {
     topStoriesContainer.removeChild(topStoriesContainer.lastChild);
   }
  },

  disableDiggMan: function diggPopup_disableDiggMan() {
    if (document.getElementById("digg-button")) {
      document.getElementById("digg-button").removeAttribute("on");
      document.getElementById("digg-button").setAttribute("disabled", "true");
    }
  },
  
  enableDiggMan: function diggPopup_enableDiggMan() {
    if (document.getElementById("digg-button")) {
      document.getElementById("digg-button").removeAttribute("disabled");
    }
  },

  getFormattedTime: function diggPopup_getFormattedTime(aInput) {
    var dateFormatter = CC["@flock.com/date-formatter;1"]
                        .getService(CI.flockIDateFormatter);
    var now = new Date();
    var seconds = Math.round(now.getTime() / 1000) - aInput;
    var minutes = Math.round(seconds / 60);
    var hours = Math.round(seconds / (60 * 60));
    var days = Math.round(seconds / (60 * 60 * 24));
    var label = null;
    if (seconds < 60) {
      label = "flock.people.mecard.lastUpdatePretty.aMomentAgo";
      return this.mSb.GetStringFromName(label);
    } else if (minutes < 60) {
      if (minutes > 1) {
        label = "flock.people.mecard.lastUpdatePretty.minutesAgo";
        return minutes + " " + this.mSb.GetStringFromName(label);
      } else {
        label = "flock.people.mecard.lastUpdatePretty.oneMinuteAgo"
        return this.mSb.GetStringFromName(label);
      }
    } else if (hours < 24) {
      if (hours > 1) {
        label = "flock.people.mecard.lastUpdatePretty.hoursAgo";
        return hours + " " + this.mSb.GetStringFromName(label);
      } else {
        label = "flock.people.mecard.lastUpdatePretty.oneHourAgo";
        return this.mSb.GetStringFromName(label);
      }
    } else if (days < 7) {
      if (days > 1) {
        label = "flock.people.mecard.lastUpdatePretty.daysAgo"
        return days + " " + this.mSb.GetStringFromName(label);
      } else {
        label = "flock.people.mecard.lastUpdatePretty.oneDayAgo";
        return this.mSb.GetStringFromName(label);
      }
    } else {
      return dateFormatter.getDateString(aInput * 1000);
    }
  },

  loadMediaBar: function diggPopup_loadMediaBar(aShortName, aQuery, aName)
  {
    var win = _wm.getMostRecentWindow("navigator:browser");
    if (win) {
      win.FlockTopbar.selectById("flock_MediabarTopbarBroadcaster", true);
      win.mediabar.loadQuery(aShortName, aQuery, aName);
    }
    diggPopup.hide();
  },

  diggIt: function diggPopup_diggIt() {
    var wm = CC["@mozilla.org/appshell/window-mediator;1"]
             .getService(CI.nsIWindowMediator);
    var win = wm.getMostRecentWindow("navigator:browser");
    if (!win) {
      return; 
    }
    var url = win.gBrowser.currentURI.spec;

    var inst = this; 
    var urlInfo_Listener = {
      onSuccess: function loc_success(aResult) {
        aResult.QueryInterface(CI.nsIPropertyBag);
        url = aResult.getProperty("diggUrl");
        document.getElementById("diggIt").src = url;
        inst.performAction("diggIt");
      },      
      onError: function loc_error(aError) {
        inst.mLogger.error("getDiggInfo_onError(..., '" + aError + ", ...)");
      }
    }
    this.mDiggService.loadDiggInfo(url, urlInfo_Listener, false); 
  },

  performAction: function diggPopup_performAction(aId) {
    var url = document.getElementById(aId).src
    if (!url) {
      url = document.getElementById(aId).getAttribute("src");
    }
    var windowManager = CC["@mozilla.org/appshell/window-mediator;1"]
                        .getService(CI.nsIWindowMediator);
    var win = windowManager.getMostRecentWindow("navigator:browser");
    if (win && url) {
      win.openUILinkIn(url, "tab");
    }
    diggPopup.hide();
    this.recordMetric(aId);
  },

  recordMetric: function diggPopup_recordMetric(aId) {
    switch (aId) {
      case "diggIt":
        this.mMetrics.report("DiggManFlyout-DiggIt-Click", "submit");
        break;

      default:
        if (aId.indexOf("topStory_") > -1) {
          this.mMetrics.report("DiggManFlyout-TopFive-Click", null);
        }
        break;
    }
  },
  
  openOptions: function diggPopup_openOptions() {
    openPreferences("paneContent");
    diggPopup.hide();
  }
};

function flock_DiggPopupInit() {
  diggPopup._init();
}

function flock_DiggPopupUninit() {
  diggPopup._uninit();
}

window.addEventListener("load", flock_DiggPopupInit, false);
window.addEventListener("unload", flock_DiggPopupUninit, false);
